unit DabBoozePlant;

interface

uses
  Classes, DabDepartment, DabEmployee, CustomVisitor;

type
  TDabBoozePlant = class(TCollectionItem)
  private
    FPlantName   : string;
    FLocation    : string;
    FDepartments : TDabDepartments;
    FEmployees   : TDabEmployees;

    procedure SetDepartments(const Value : TDabDepartments);
    procedure SetEmployees(const Value : TDabEmployees);
  protected
  public
    constructor Create(Collection : TCollection); override;
    destructor  Destroy; override;

    procedure Assign(Source : TPersistent); override;
    // Needed for Visitor
    procedure Accept(Visitor : TCustomVisitor); virtual;
  published
    property PlantName   : string read FPlantName write FPlantName;
    property Location    : string read FLocation write FLocation;
    property Departments : TDabDepartments read FDepartments write SetDepartments;
    property Employees   : TDabEmployees read FEmployees write SetEmployees;
  end;


  TDabBoozePlants = class(TOwnedCollection)
  private
  protected
    function  GetItem(Index : Integer) : TDabBoozePlant;
    procedure SetItem(Index : Integer;Value : TDabBoozePlant);
  public
    function  Add : TDabBoozePlant;
    procedure AddNew(const APlantName,ALocation : string);
    // Needed for Visitor
    procedure Accept(Visitor : TCustomVisitor); virtual;
    function  FindItemID(ID : Integer) : TDabBoozePlant;
    function  Insert(Index : Integer) : TDabBoozePlant;

    property Items[Index : Integer] : TDabBoozePlant read GetItem
                                                     write SetItem; default;
  published
  end;


implementation

uses
  SysUtils;

{ TDabBoozePlant }

constructor TDabBoozePlant.Create(Collection : TCollection);
begin
  inherited;
  FDepartments := TDabDepartments.Create(Self,TDabDepartment);
  FEmployees   := TDabEmployees.Create(Self,TDabEmployee);
end;


destructor TDabBoozePlant.Destroy;
begin
  FreeAndNil(FDepartments);
  FreeAndNil(FEmployees);
  inherited;
end;


procedure TDabBoozePlant.SetDepartments(const Value : TDabDepartments);
begin
  FDepartments.Assign(Value);
end;


procedure TDabBoozePlant.SetEmployees(const Value : TDabEmployees);
begin
  FEmployees.Assign(Value);
end;


procedure TDabBoozePlant.Assign(Source : TPersistent);
begin
  if Source is TDabBoozePlant then begin
    Departments := TDabBoozePlant(Source).Departments;
    Employees   := TDabBoozePlant(Source).Employees;
  end;
end;


procedure TDabBoozePlant.Accept(Visitor : TCustomVisitor);
begin
  Visitor.Visit(Self);
  Employees.Accept(Visitor);
  Departments.Accept(Visitor);
end;


{ TDabBoozePlants }

function TDabBoozePlants.GetItem(Index : Integer) : TDabBoozePlant;
begin
  Result := TDabBoozePlant(inherited GetItem(Index));
end;


procedure TDabBoozePlants.SetItem(Index : Integer;Value : TDabBoozePlant);
begin
  inherited SetItem(Index,Value);
end;


function TDabBoozePlants.Add : TDabBoozePlant;
begin
  Result := TDabBoozePlant(inherited Add);
end;


procedure TDabBoozePlants.AddNew(const APlantName,ALocation : string);
begin
  with Add do begin
    PlantName := APlantName;
    Location  := ALocation;
  end;
end;


procedure TDabBoozePlants.Accept(Visitor : TCustomVisitor);
  var
    i : Integer;
begin
  Visitor.Visit(Self);

  for i := 0 to Count - 1 do begin
    Items[i].Accept(Visitor);
  end;
end;


function TDabBoozePlants.FindItemID(ID : Integer) : TDabBoozePlant;
begin
  Result := TDabBoozePlant(inherited FindItemID(ID));
end;


function TDabBoozePlants.Insert(Index : Integer) : TDabBoozePlant;
begin
  Result := TDabBoozePlant(inherited Insert(Index));
end;


end.
